#!/usr/bin/perl -w
 
use strict;
use warnings;

# TIGR Modules
use TIGR::Foundation;

my $tigr_tf = new TIGR::Foundation;
my $PRG = $tigr_tf->getProgramInfo('name');
my $REV="1.0";
my @DEPENDS=("TIGR::Foundation");

# help info
my $HELPTEXT = qq~
Program that computes the N50 and Esize statistics for the GAGE assemblies. 

Usage: $PRG file [options] 
	  
  file:	tab/space delimited file, in which column "i" contains the sequence lengths.
  
  options:

  	-min		- min length
	-max		- max length
	-i		- column number (starts from 0)	
	-t		- description   (to print in the output)
	-g 		- genome size   (for N50 computation)

	-h|help		- Print this help and exit;
	-V|version	- Print the version and exit;
	-depend		- Print the program and database dependency list;
	-debug <level>	- Set the debug <level> (0, non-debug by default); 
 
~;

my $MOREHELP = qq~
Return Codes:   0 - on success, 1 - on failure.
~;

# Configure TIGR Foundation
$tigr_tf->setHelpInfo($HELPTEXT.$MOREHELP);
$tigr_tf->setUsageInfo($HELPTEXT);
$tigr_tf->setVersionInfo($REV);
$tigr_tf->addDependInfo(@DEPENDS);

###############################################################################
#
# Main program
#
###############################################################################

MAIN:
{
	my $i=0;

	my $sump;	
	my $n50=0;	
	my %count;
	my $count=0;
	my @keys;
	my ($min,$max);
	my $text=".";		
	my $genomeSize;
	my $Esize;
	
	# validate input parameters
	my $result = $tigr_tf->TIGR_GetOptions(
		"min=s"		=>	\$min,
		"max=s"		=>	\$max,
		"i=i"		=> 	\$i,
		"text=s"	=>	\$text,
		"genomeSize=i"	=>	\$genomeSize,
	);
	$tigr_tf->printUsageInfoAndExit() if (!$result or !defined($genomeSize));
	
	# parse input
	while(<>)
	{
		next if(/^$/ or /^#/);
		chomp;
		my @f=split;
		next unless(@f);
		
		$tigr_tf->bail("Undefined column $i in $_") if(!defined($f[$i]));
		
		my $key=$f[$i];				# should be numeric
		$key=~/^\d+$/ or  /^[+-]?\d+(\.\d+)?$/ or $tigr_tf->bail("Coulumn $i must be numeric in line $.: $_\n");		

		next if(defined($min) and $key<$min);
		next if(defined($max) and $key>$max);
		
		$count{$key}++;
		$count++;
	}
	
	# exit if no elements	
	exit 1 unless($count) ;
		
	@keys=reverse (sort {$a <=> $b} keys %count);
	# get n50 	
	$sump=0;
	foreach my $key (@keys)
	{
		$n50=$key;
		$sump+=$key*$count{$key};
		last if($sump>$genomeSize/2);
	}
	
	# get Esize
	$sump=0;
        foreach my $key (@keys)
        {
		$sump+=$key*$key*$count{$key};
	}			
        $Esize=($sump/($genomeSize));
		
	printf ("%-20s %-10s %-10s %-10s\n",".","elem","n50","Esize");			
	printf ("%-20s %-10i %-10i %-10i\n",$text,$count,$n50,$Esize);

	exit 0;
}

